﻿App = { mongocrud: {} };
var mongoose = require('mongoose');
var ObjectId = mongoose.Schema.Types.ObjectId;

module.exports = {
	init: function (app, settings, authenticate) {
		
		if (settings.connection != null)
			mongoose.connect(settings.connection);
		
		App = app;
		App.mongocrud = {};
		
		apiDomain = settings.apidomain == null ? 'api' : settings.apidomain;
		// Authenticate has {role: role, call:  } (get,get-all,add,update,remove)
		authenticate = authenticate == null ? function (a, c) { c(true); } : authenticate;
		App.authenticate = authenticate;
	},
	ObjectId: ObjectId,
	clean: function (obj) {
		var newobj = {};
		for (var param in obj) {
			if (param.indexOf('_') != 0) {
				newobj[param] = obj[param];
			}
		}
		return newobj;
	},
	model: function (name, model) {
		if (model == null) return App.mongocrud[name];
		
		model.DateCreated = Date;
		model.DateUpdated = Date;
		model.CreatedByUser = Object;
		model.ModifiedByUser = Object;
		model.Active = Boolean;
		var _model = mongoose.model(name, mongoose.Schema(model));;
		
		App.mongocrud[name] = {
			model: _model,
			new: function (data, req) {
				var req = req == null ? { body: {} } : req,
					body = req.body;
				var _data = {
					CreatedByUser : req.user,
					ModifiedByUser: req.user,
					DateCreated   : new Date(),
					DateUpdated   : new Date(),
					Active        : body.Active == null ? true : body.Active
				};
				for (var param in data) {
					if (param.indexOf("_") != 0) {
						_data[param] = data[param];
					}
				}
				return new _model(_data);
			},            
			push: function (array, relation, callback) {
				var relation = relation,
					array = array,
					callback = callback;
				
				var model = relation.model == null ? _model:relation.model;
				relation.data.save(function (err, transaction) {
					if (err) return callback(err, null);
					var pushdata = { $push: {} };
					pushdata.$push[array] = transaction._id;
					
					model.update({ _id: relation._id }, pushdata, function (err, data) {
						if (err) return callback(err, null);
						callback(null, data);
					});
				});
			}

		}
		
		
		return {
			crud: function (crud) {
				crud = crud == null ? {}                   : crud;
				crud.url = crud.url == null ? "/" + name   : crud.url;
				crud.role = crud.role == null ? 'none'     : crud.role;
				crud.exclude = crud.exclude == null ? []   : crud.exclude;
				crud.populate = crud.populate == null ? {} : crud.populate;
				crud.custom = crud.custom == null ? {}     : crud.custom;
				module.exports.crud(_model, crud);
			},
			model: _model,
			name: name
		};
	},
	error: function (res, err, data) {
		res.status(500);
		res.end(JSON.stringify({ error: err, data: data }))
	},    
	crud: function (Model, crud) {
		var model = Model;
		var modelBase = module.exports.model(Model.modelName);
		var error = module.exports.error;
		
		crud.exclude = crud.exclude == null ? [] : crud.exclude;
		crud.populate = crud.populate == null ? {} : crud.populate;
		crud.url = "/" + apiDomain + crud.url;
		
		var validate = function (call, then) {
			call.res.setHeader('Content-Type', 'application/json');
			var then = then;
			call.role = crud.role;
			call.url = crud.url;
			call.model = model;
			
			App.authenticate(call, function (validated) {
				if (validated) {
					
					then(call);
				} else {
					res.end(JSON.stringify({ error: "Authentication Failed", call: url, model: object.name }));
				}
			})
		}
		var populate = function (search, call) {
			var populate = crud.populate[call];
			if (populate == null) populate = [];
			for (var i = 0; i < populate.length; i++) {
				search.populate(populate[i], null, { Active: true });
			}
		}
		
		var update = function (req, res, call) {
			validate({ call: 'update', req: req, res: res }, function (params) {
				var model = req.body;
				var _id = model._id;
				delete model._id;
				for (var prop in model) {
					if (prop.indexOf('_') != -1)
						delete model[prop];
				}
				
				model.ModifiedByUser = params.user;
				model.DateUpdated = new Date();
				model.Active = model.Active == null ? true : model.Active;
				
				Model.update({ _id: _id }, { $set: model }, function (err, data) {
					if (err) return error(res, err, data);
					res.end(JSON.stringify(data));
				});
			});
		}
		
		if (crud.exclude.indexOf('find-one') == -1)
			App.post(crud.url + '/find-one', function (req, res) {
				var req = req,
					res = res;
				validate({ call: 'find-one', req: req, res: res }, function () {
					params = req.body;
					var hasparams = false;
					for (var param in params) {
						if (param.indexOf('_') != 0)
							hasparams = true;
					}
					if (!hasparams) {						
						return error(res, { error: crud.url + '/find-one requires at least one parameter to match a record' }, params);
					}
					
					var search = Model.findOne(params);
					populate(search, 'find-one');
					search.exec(function (err, data) {
						if (err) return error(res, err, data);
						res.end(JSON.stringify(data));
					});
				});
			});
		
		if (crud.exclude.indexOf('find') == -1)
			App.post(crud.url + '/find', function (req, res) {
				validate({ call: 'find', req: req, res: res }, function (params) {
					params = req.body;
					params.Active = params.Active == null ? true:params.Active;
					var search = Model.find(params);
					populate(search, 'find');
					search.exec(function (err, data) {
						if (err) return error(res, err, data);
						res.end(JSON.stringify(data));
					});
				});
			});
		
		if (crud.exclude.indexOf('add') == -1)
			App.post(crud.url + '/add', function (req, res) {
				validate({ call: 'add', req: req, res: res }, function (params) {
					var body = JSON.parse(JSON.stringify(req.body));
					var data = {
						CreatedByUser : params.user,
						ModifiedByUser: params.user,
						DateCreated   : new Date(),
						DateUpdated   : new Date(),
						Active        : body.Active == null ? true : body.Active
					};
					
					for (var prop in body) {
						if (prop.indexOf('-') == -1)
							data[prop] = body[prop];
					}
					
					var model = new Model(data);
					
					model.save(function (err, data) {
						if (err) return error(res, err, data);
						res.end(JSON.stringify(data));
					});
				});
			});
		
		if (crud.exclude.indexOf('update') == -1)
			App.post(crud.url + '/update', function (req, res) {
				update(req, res, 'update');
			});
		
		if (crud.exclude.indexOf('remove') == -1)
			App.post(crud.url + '/remove', function (req, res) {
				req.body = {
					_id: req.body._id,
					Active: false
				};
				update(req, res, 'remove');
			});
		
		for (var param in crud.custom) {
			if (param.indexOf('_') != 0) {
				var value = crud.custom[param];
				if (typeof value == 'function') {
					App.post(crud.url + '/' + param, function (req, res) {
						var req = req,
							res = res;
						validate({ call: param, req: req, res: res }, function (params) {
							res.json = function (data) {
								res.end(JSON.stringify(data));
							}
							value(req, res, modelBase);
						});
					});
				}
			}
		}
	},
};
